// Defines the entry point for the DLL application.
#include "stdafx.h"
#include "resource4.h"
#include <strsafe.h>
#include <stdlib.h>

// Global Variables:
HINSTANCE hInst;		// current instance
HWND hDialog1;			// window handle
HINSTANCE gPlugId;		// global plugin id


BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{
	hInst = (HINSTANCE)hModule;
	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	case DLL_PROCESS_DETACH:
		break;
	}
	return TRUE;
}

VOID ShowMsg(char* s)
{
	MessageBoxA(
			NULL,
			s,
			"Message: ",
			MB_ICONWARNING | MB_OK | MB_DEFBUTTON1
		);
}

// declare api function from external editplug exe api
extern "C" VOID __cdecl epSetActiveEdSelText(char* p);


char* inttostr(int i)
{
	char buffer [15];
	itoa (i,buffer,10);
	return buffer;
}

VOID SayHello()
{
	ShowMsg("Hello World");
}

VOID SayHelloInEditor()
{
	// call editplug api function
	epSetActiveEdSelText(" Hello Text Editor ");
}


// Message handler for dialog box.
LRESULT CALLBACK Dialog1(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	hDialog1 = hDlg; // set global variable
	switch (message)
	{
		case WM_INITDIALOG:
				return TRUE;

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) 
			{
				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;
			}
			if (LOWORD(wParam) == IDC_BUTTON1) 
			{
				SayHello();
			}
			if (LOWORD(wParam) == IDC_BUTTON2) 
			{
				SayHelloInEditor();
			}
			break;
	}
    return FALSE;
}

void LastErr(LPTSTR lpszFunction)
{
    DWORD dw;
    LPVOID lpMsgBuf;
    LPVOID lpDisplayBuf;
	dw = GetLastError(); 
    FormatMessage(
        FORMAT_MESSAGE_ALLOCATE_BUFFER | 
        FORMAT_MESSAGE_FROM_SYSTEM |
        FORMAT_MESSAGE_IGNORE_INSERTS,
        NULL,
        dw,
        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
        (LPTSTR) &lpMsgBuf,
        0, NULL );
    lpDisplayBuf = (LPVOID)LocalAlloc(LMEM_ZEROINIT, 
        (lstrlen((LPCTSTR)lpMsgBuf) + lstrlen((LPCTSTR)lpszFunction) + 40) * sizeof(TCHAR)); 
    StringCchPrintf((LPTSTR)lpDisplayBuf, 
        LocalSize(lpDisplayBuf) / sizeof(TCHAR),
        TEXT("%s failed with error %d: %s"), 
        lpszFunction, dw, lpMsgBuf); 
    MessageBox(NULL, (LPCTSTR)lpDisplayBuf, TEXT("Error"), MB_OK); 
}

int CreatePlugWin()
{
	HWND dialogh;
	dialogh = CreateDialog(hInst, MAKEINTRESOURCE(IDD_DIALOG1), NULL, (DLGPROC)Dialog1);
	if (dialogh == NULL) {
		ShowMsg("dialog handle is null");
		LastErr(TEXT("CreateDialog"));
	} else {
		ShowWindow(dialogh, SW_SHOW);
  	}
	return 1;
}

// function to show plugin window, exported from dll
HWND __cdecl plgShow() 
{
    // if the window exists, show it, if not, create it
	if (IsWindow(hDialog1)) {
		ShowWindow(hDialog1, SW_SHOW);
		SetForegroundWindow(hDialog1);
	} else {
		CreatePlugWin();
	}
	return hDialog1;
}

// free plugin resources, exported from dll
VOID __cdecl plgWinFree() 
{
	DestroyWindow(hDialog1);
}

// return main window handle of plugin, exported from dll
HWND __cdecl plgGetWinHandle()
{
	HWND result;
	result = 0;
	if (IsWindow(hDialog1)) {
		result = hDialog1;
	}
	return result;
}

// set up global plugin id for later use
VOID __cdecl plgSetId(HINSTANCE id)
{
	gPlugId = id;
}